import os
import codecs
import datetime
from django.utils.safestring import mark_safe
import markdown
import yaml


class NotFound(Exception):
    def __init__(self, message):
        super(NotFound, self).__init__("Missing: %s" % message)


class Resource(object):
    """
    An object to represent a resource at a specific url

    maps to a file on disk and renders it from markdown to html
    """
    renderer = markdown.Markdown()

    @classmethod
    def locate(cls, path):
        if not path or path.endswith('/'):
            path += 'index'
        path += '.txt'
        if os.path.exists(path):
            return cls(path)
        else:
            raise NotFound(path)

    def __init__(self, path):
        self.path = path
        data = codecs.open(path, encoding='utf-8').read()
        try:
            meta, self.source = data.split('---\n', 1)
            self.meta = yaml.load(meta)
        except ValueError:
            self.meta = {}
            self.source = data

    def body(self):
        """
        Convert the source of the page from markdown to html
        """
        return mark_safe(self.renderer.convert(self.source))

    def __getitem__(self, key):
        return self.meta[key]

    def title(self):
        return self.meta.get('title') or self.meta.get('heading')

    def date(self):
        """
        If the page data contains a date use that otherwise use
        the mtime of the file
        """
        if 'date' in self.meta:
            return self.meta['date']
        else:
            return datetime.datetime.fromtimestamp(
                os.stat(self.path).st_mtime)



import os
import time
from django.http import HttpResponseNotFound
from django.shortcuts import render_to_response
from django.utils.http import http_date
from .models import Resource, NotFound


def root(cms_root):
    """
    A view that will render a basic site based on the
    directory layout of the provided path
    """
    def view(request, path):
        try:
            page = Resource.locate(os.path.join(cms_root, path))
        except NotFound as e:
            return HttpResponseNotFound(str(e))
        response = render_to_response('bland/resource.html',
            {'page': page})
        response['Last-Modified'] = _http_date(page.date())
        return response
    return view


def _http_date(datetime):
    """
    the ludicrous amount of work required in python date/time libs
    """
    return http_date(time.mktime(datetime.timetuple()))



#!/usr/bin/env python

from distutils.core import setup


setup(name="django-bland",
    version="0.1.0",
    description="A terribly basic file based CMS for django",
    author="Adam Sven Johnson",
    author_email="adam@pkqk.net",
    url="https://github.com/pkqk/django-bland",
    license='MIT',
    packages=["bland"],
    package_data={
        "bland": ['templates/bland/*.html', 'static/bland/*.css']
    },
    requires=[
        'Django',
        'pyyaml',
        'markdown'
    ]
)
